/* This is "newton6.c", written to accompany Introduction to
   Scientific Programming by Joseph L. Zachary. */

/* This program is an implementation and a test driver for
   Newton's method as described in Chapter 14 of the text. */


#include <stdio.h>
#include <math.h>



/* This is x^4 + 2x^3 - 14x^2 + 2x + 1, whose root we seek. */

double f (double x)
{
  return(1 + x*(2 + x*(-14 + x*(2 + x))));
}



/* This is 4x^3 + 6x^2 - 28x + 2, which is the first
   derivative of f. */

double fprime (double x)
{
  return(2 + x*(-28 + x*(6 + x*4)));
}



/* An implementation of Newton's method.  It improves the
   guess "g0" for the root of "f" until the absolute value
   of the difference between two successive guesses becomes
   small relative to the first of the two guesses.  It then
   returns the improved guess. */

double newton (double g0)
{
  double oldg0;

  do {
    oldg0 = g0;
    g0 = g0 - f(g0)/fprime(g0);
  }
  while (fabs((g0-oldg0)/g0) > 1e-10);

  return(g0);
}



/* A test driver that uses "newton" to find a root of "f"
   given a guess read from the keyboard. */

void main (void)
{
  double guess, root;
  printf("Please enter an initial guess: ");
  scanf("%lf", &guess);
  root = newton(guess);
  printf("An approximate root is %.10g\n", root);
  printf("f(approximate root) is %.10g\n", f(root));
}
