/* This is "newton9.c", written to accompany Introduction to
   Scientific Programming by Joseph L. Zachary. */

/* Together with "complex.h" and "complex.c", this is an
   implementation and test driver for Newton's method with complex
   numbers as described in Chapter 16 of the text. */


#include <stdio.h>
#include <math.h>
#include "complex.h"



/* This is x^2 - 6x + 13, whose root we seek. */

complex f (complex x)
{
  complex c6, c13;

  c6 = make(6,0);
  c13 = make(13, 0);

  return(add(sub(mul(x,x), mul(c6,x)), c13));
}



/* This is 2x - 6, which is the first derivative of f. */

complex fprime (complex x)
{
  complex c2, c6;

  c2 = make(2,0);
  c6 = make(6,0);

  return(sub(mul(c2,x), c6));
}



/* An implementation of Newton's method that finds complex roots.
   It improves the guess "g0" for the root of "f" until the absolute
   value of the difference between two successive guesses becomes
   small relative to the first of the two guesses.  It then returns
   the improved guess. */

complex newton (complex g0)
{
  complex oldg0;   /* The value of g0 during the previous iteration. */

  do {
    oldg0 = g0;
    g0 = sub(g0, div(f(g0), fprime(g0)));
  }
  while (absval(div(sub(g0, oldg0), g0)) > 1e-10);

  return(g0);
}



/* A test driver that uses "newton" to find a complex root
   of "f" given a guess read from the keyboard. */

void main (void)
{
  complex guess, root;

  printf("Please enter an initial guess: ");
  guess = read();
  root = newton(guess);
  printf("An approximate root is ");
  write(root);
  printf("\n");
}
